@@ -75,12 +75,20 @@ class BoundingBox:
         size = bbox_tuple[3:6]
         return BoundingBox.from_tuple2((topleft, size))
 
+    @staticmethod
+    def from_csv(csv_bbox: str) -> "BoundingBox":
+        bbox_tuple = tuple(int(x) for x in csv_bbox.split(","))
+        return BoundingBox.from_tuple6(bbox_tuple)
+
     @staticmethod
     def from_auto(obj) -> "BoundingBox":
         if isinstance(obj, BoundingBox):
             return obj
         elif isinstance(obj, str):
-            return BoundingBox.from_auto(json.loads(obj))
+            if ":" in obj:
+                return BoundingBox.from_auto(json.loads(obj))
+            else:
+                return BoundingBox.from_csv(obj)
         elif isinstance(obj, dict):
             return BoundingBox.from_wkw(obj)
         elif isinstance(obj, BoundingBoxNamedTuple):
@@ -120,6 +128,10 @@ class BoundingBox:
 
         return tuple(self.topleft.tolist() + self.size.tolist())  # type: ignore
 
+    def as_csv(self) -> str:
+
+        return ",".join(map(str, self.as_tuple6()))
+
     def __repr__(self) -> str:
 
         return "BoundingBox(topleft={}, size={})".format(
@@ -181,20 +193,40 @@ class BoundingBox:
 
         return not all(self.size > 0)
 
-    def in_mag(self, mag: Mag, ceil: bool = False) -> "BoundingBox":
+    def in_mag(self, mag: Mag) -> "BoundingBox":
 
         np_mag = np.array(mag.to_array())
 
-        def ceil_maybe(array: np.ndarray) -> np.ndarray:
-            if ceil:
-                return np.ceil(array)
-            return array
+        assert (
+            np.count_nonzero(self.topleft % np_mag) == 0
+        ), f"topleft {self.topleft} is not aligned with the mag {mag}. Use BoundingBox.align_with_mag()."
+        assert (
+            np.count_nonzero(self.bottomright % np_mag) == 0
+        ), f"bottomright {self.bottomright} is not aligned with the mag {mag}. Use BoundingBox.align_with_mag()."
 
         return BoundingBox(
-            topleft=ceil_maybe(self.topleft / np_mag).astype(np.int),
-            size=ceil_maybe(self.size / np_mag).astype(np.int),
+            topleft=(self.topleft // np_mag).astype(np.int),
+            size=(self.size // np_mag).astype(np.int),
         )
 
+    def align_with_mag(self, mag: Mag, ceil=False):
+        """Rounds the bounding box, so that both topleft and bottomright are divisible by mag.
+
+        :argument ceil: If true, the bounding box is enlarged when necessary. If false, it's shrinked when necessary.
+        """
+
+        np_mag = np.array(mag.to_array())
+
+        align = lambda point, round_fn: round_fn(point / np_mag).astype(np.int) * np_mag
+
+        if ceil:
+            topleft = align(self.topleft, np.floor)
+            bottomright = align(self.bottomright, np.ceil)
+        else:
+            topleft = align(self.topleft, np.ceil)
+            bottomright = align(self.bottomright, np.floor)
+        return BoundingBox(topleft, bottomright - topleft)
+
     def contains(self, coord: Shape3D) -> bool:
 
         coord = np.array(coord)
@@ -204,7 +236,7 @@ class BoundingBox:
         )
 
     def contains_bbox(self, inner_bbox: "BoundingBox") -> bool:
-        return inner_bbox.intersected_with(self) == inner_bbox
+        return inner_bbox.intersected_with(self, dont_assert=True) == inner_bbox
 
     def chunk(
         self, chunk_size: Shape3D, chunk_border_alignments: Optional[List[int]] = None
@@ -256,6 +288,13 @@ class BoundingBox:
             self.topleft[2] : self.bottomright[2],
         ]
 
+    def as_slices(self) -> Tuple[slice, slice, slice]:
+        return np.index_exp[
+            self.topleft[0] : self.bottomright[0],
+            self.topleft[1] : self.bottomright[1],
+            self.topleft[2] : self.bottomright[2],
+        ]
+
     def copy(self) -> "BoundingBox":
 
         return BoundingBox(self.topleft.copy(), self.bottomright.copy())
